"""
Migration linter for Django app.

This app has `django` management command
that efficiently handle MRs with many migrations.
"""

default_app_config = "migration_lint.django.apps.MigrationLintDjangoConfig"
